/*
** Module   : LockManager.java
** Abstract : Lock manager interface
**
** Copyright (c) 2004-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ------------------Description-------------------
** 001 ECF 20051013   @23075 Created initial version. Defines interface
**                           for a pessimistic lock manager implementation
**                           which supports the Progress locking semantic.
** 002 ECF 20051104   @23240 Added queryLock method. Queries lock type
**                           held by the current context on a specific
**                           record.
** 003 ECF 20051110   @23380 Removed database from lock and queryLock
**                           signatures. Added setDatabase method to allow
**                           database name to be preserved for logging. It
**                           is unnecessary to track database name with
**                           each lock, since a LockManager instance is
**                           now database-specific.
** 004 ECF 20051207   @23754 Use more specific exception type if lock is
**                           unavailable. PersistenceException replaced
**                           with LockUnavailableException.
** 005 ECF 20070412   @32981 Changed API to use RecordIdentifier objects.
**                           Previously used separate parameters for table
**                           and primary key.
** 006 ECF 20070911   @35289 Replaced database name string with Database
**                           information object. Required for non-local
**                           database connections.
** 007 ECF 20090801   @43500 Added support for lock administration. Added
**                           getLockAdministrator() method, which should be
**                           implemented to verify admin access, then return a
**                           LockAdministrator instance.
** 008 VMN 20130802          Added lockTable method.
*/

package com.goldencode.p2j.persist;

/**
 * Interface which must be implemented by classes which provide the Progress
 * semantic of pessimistic, row-level, database locking.  Provides services
 * to lock a database record in the current context, and to query the lock
 * type currently held in the current context on a specific record.
 */
public interface LockManager
{
   /**
    * Set the name of the physical database with which this lock manager is
    * associated.
    *
    * @param   database
    *          Database information object.
    */
   public void setDatabase(Database database);
   
   /**
    * Retrieve the {@link LockAdministrator} object associated with this
    * <code>LockManager</code> instance.
    * <p>
    * To ensure a caller has suffient rights to access administrative lock
    * functions, the implementation of this method should confirm with the
    * administrative server before returning any lock information.
    * 
    * @return  <code>LockAdministrator</code> instance.
    */
   public LockAdministrator getLockAdministrator();
   
   /**
    * Attempt to obtain the specified lock type on a particular record.  If
    * the lock cannot be obtained immediately, the current thread will block
    * until the lock becomes available and is obtained, unless the requested
    * lock type is a "no-wait" lock.  In the latter case, an exception is
    * raised, indicating that the lock is not available.  The current thread
    * continues once the lock is obtained.
    * <p>
    * A lock currently held can be released by specifying
    * <code>LockType.NONE</code> for the <code>lockType</code> parameter.
    * <p>
    * The normal return of this method indicates that the lock request (or
    * lack thereof) completed successfully, and that the current context now
    * holds a lock of the requested type.
    *
    * @param   lockType
    *          Type of lock to be obtained.  <code>LockType.NONE</code> is
    *          used to release an existing lock, and to continue with no
    *          lock.
    * @param   ident
    *          ID which uniquely identifies the record being locked.
    * @param   update
    *          <code>true</code> if the lock state for the current record
    *          should be updated;  else <code>false</code>.
    *
    * @throws  LockUnavailableException
    *          if a "no-wait" lock cannot be acquired immediately.
    */
   public void lock(LockType lockType, RecordIdentifier ident, boolean update)
   throws LockUnavailableException;
   
   /**
    * Query the lock type currently held by the current context for the
    * specified database record.
    * <p>
    * This method should always return the <b>non-</b><code>NO_WAIT</code>
    * variants of a lock type.  That is, even if the actual lock type is
    * <code>LockType.EXCLUSIVE_NO_WAIT</code> or
    * <code>LockType.SHARE_NO_WAIT</code>, the simpler types of
    * <code>LockType.EXCLUSIVE</code> and <code>LockType.SHARE</code>,
    * respectively, should be returned.  The lack of a lock is required to
    * return <code>LockType.NONE</code> rather than <code>null</code>.
    *
    * @param   ident
    *          ID which uniquely identifies the record being queried.
    *
    * @return  The lock type currently held by the current context, never
    *          <code>null</code>.
    */
   public LockType queryLock(RecordIdentifier ident);

   /**
    * Attempt to obtain the specified lock type on a table. If the lock cannot be obtained
    * immediately, the current thread will block until the lock becomes available and is
    * obtained, unless the requested lock type is a "no-wait" lock.  In the latter case,
    * an exception is raised, indicating that the lock is not available. The current thread
    * continues once the lock is obtained.
    * <p>
    * A lock currently held can be released by specifying
    * <code>LockType.NONE</code> for the <code>lockType</code> parameter.
    * <p>
    * Method returns previous lock type (so that calling code knows what to set it back to after
    * bulk delete).
    * The normal return of this method indicates that the lock request (or lack thereof)
    * completed successfully, and that the current context now holds a lock of the requested
    * type.
    *
    * @param   lockType
    *          Type of lock to be obtained.  <code>LockType.NONE</code> is used to release an
    *          existing lock, and to continue with no lock.
    * @param   table
    *          Table name.
    * @param   update
    *          <code>true</code> if the lock state for the current table should be updated;
    *          else <code>false</code>.
    *
    * @throws  LockUnavailableException
    *          if a "no-wait" lock cannot be acquired immediately.
    *
    * @return  previous lock type.
    */
   public LockType lockTable(String table, LockType lockType, boolean update)
   throws LockUnavailableException;
}
